--
-- ManualUnloading Script
-- author: Thomas Heck (Patar)
-- date: 04.12.2016
--
-- V1

print("--- ManualUnloading by Patar loaded! (V1)")
ManualUnloading = {};

function ManualUnloading.prerequisitesPresent(specializations)
    return true
end;

function ManualUnloading:load(xmlFile)
	self.setUnloadingParam = ManualUnloading.setUnloadingParam;

	if self:getCapacity() > 0 then
		self.canUnload = false;
		self.unloadingStarted = false;
	end;
end;

function ManualUnloading:writeStream(streamId, connection)
	if self:getCapacity() > 0 and self.hasPipe then
		if self.canUnload then
			 streamWriteInt8(streamId, 1);
		else 
			 streamWriteInt8(streamId, 2);
		end;
		if self.unloadingStarted then
			 streamWriteInt8(streamId, 3);
		else 
			 streamWriteInt8(streamId, 4);
		end;
	end;
end;

function ManualUnloading:readStream(streamId, connection)
	if self:getCapacity() > 0 and self.hasPipe then
		local canUnload = streamReadInt8(streamId);
		local unloadingStarted = streamReadInt8(streamId)
		
		self:setUnloadingParam(canUnload, true);
		self:setUnloadingParam(unloadingStarted, true);
	end;
end;

function ManualUnloading:delete()
end;

function ManualUnloading:mouseEvent(posX, posY, isDown, isUp, button)
end;

function ManualUnloading:keyEvent(unicode, sym, modifier, isDown)
end;

function ManualUnloading:update(dt)
	if self.hasPipe and self:getCapacity() > 0 then
		if self.isServer then
			canUnload = true;
			
			if self.foldAnimTime ~= nil and (self.foldAnimTime > self.pipeFoldMaxTime or self.foldAnimTime < self.pipeFoldMinTime) then
				canUnload = false;
			end;
			if not self.pipeStateIsUnloading[self.pipeCurrentState] then
				canUnload = false;
			end;
			if not self.dischargeToGround and self.trailerFound == 0 and self.shovelTargetFound == nil then
				canUnload = false;
			end;
			
			if canUnload ~= self.canUnload then
				if canUnload then
					self:setUnloadingParam(1);
				else
					self:setUnloadingParam(2);
				end;
			end;
			
		end;
		
		if self:getIsActiveForInput(false) and self.canUnload then
			if InputBinding.hasEvent(InputBinding.ACTIVATE_OBJECT) then
				if self.unloadingStarted then
					self:setUnloadingParam(4);
				else
					self:setUnloadingParam(3);
				end;
			end;
		end;
		
		if not self.canUnload and self.unloadingStarted and not self.isHired then
			self:setUnloadingParam(4);
		end;
		
		if self.isHired and not self.unloadingStarted then
			self:setUnloadingParam(3);
		end;
	end;
end;

function ManualUnloading:updateTick(dt)
end;

function ManualUnloading:draw()
	if self:getCapacity() > 0 and self.hasPipe and self.canUnload then
		if not self.unloadingStarted then
			g_currentMission:addHelpButtonText(g_i18n:getText("MU_startUnloading"), InputBinding.ACTIVATE_OBJECT);
		else
			g_currentMission:addHelpButtonText(g_i18n:getText("MU_stopUnloading"), InputBinding.ACTIVATE_OBJECT);
		end;
	end;
end;

function ManualUnloading:onAttach(attacherVehicle)
end;

function ManualUnloading:onDetach()
end;

function ManualUnloading:getIsOverloadingAllowedNew(superFunc)

	if self.hasPipe and self:getCapacity() > 0 then
		if self.foldAnimTime ~= nil and (self.foldAnimTime > self.pipeFoldMaxTime or self.foldAnimTime < self.pipeFoldMinTime) then
			return false;
		end;

		if not self.pipeStateIsUnloading[self.pipeCurrentState] then
			return false;
		end;

		if not self.dischargeToGround and self.trailerFound == 0 and self.shovelTargetFound == nil then
			return false;
		end;
		
		if not self.unloadingStarted then
			return false;
		end;
	end;
	
	return true;
end;

function ManualUnloading:setUnloadingParam(unloadingValue, noEventSend)

	-- 1 = canUnload true
	-- 2 = canUnload false
	-- 3 = unloadingStarted true
	-- 4 = unloadingStarted false
	
	if unloadingValue == 1 then
		self.canUnload = true;
	elseif unloadingValue == 2 then
		self.canUnload = false;
	elseif unloadingValue == 3 then
		self.unloadingStarted = true;
	elseif unloadingValue == 4 then
		self.unloadingStarted = false;
	end;
	
	setUnloadingParamEvent.sendEvent(self, unloadingValue, noEventSend);
end;

Pipe.load  = Utils.prependedFunction(Pipe.load, ManualUnloading.load);
Pipe.writeStream  = Utils.appendedFunction(Pipe.writeStream, ManualUnloading.writeStream);
Pipe.readStream  = Utils.appendedFunction(Pipe.readStream, ManualUnloading.readStream);
Pipe.update  = Utils.appendedFunction(Pipe.update, ManualUnloading.update);
Pipe.draw  = Utils.appendedFunction(Pipe.draw, ManualUnloading.draw);
Pipe.getIsOverloadingAllowed  = Utils.overwrittenFunction(Pipe.getIsOverloadingAllowed, ManualUnloading.getIsOverloadingAllowedNew);
